/*
 * A test of executing code in new tasks and threads.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <fsclient/process.h>
#include <systypes/format.h>

#include <stdio.h>



static long test_process(int argc, const char *argv[])
{
  process_t *process;

  /* Start the process. */

  long err = process_spawn(argc, argv, &process);

  if (err)
  {
    printf("Could not start process: %s\n", l4sys_errtostr(err));
    return err;
  }

  printf("Finished program initiation.\n");

  /* Wait for a signal from the process. */

  notify_flags_t flags;
  notify_values_t values;

  err = process_wait(process, &flags, &values);

  if (err)
  {
    printf("Could not wait for process: %s\n", l4sys_errtostr(err));
    return err;
  }

  printf("End process (flags %" pFMTnotify_flags "x values: %ld, %ld)\n", flags, values.sig, values.val);
  return L4_EOK;
}



int main(int argc, char *argv[])
{
  long err;

  if (argc < 2)
  {
    printf("Need the program to run.\n");
    return 1;
  }

  /* Obtain the common notifier. */

  notifier_t *notifier = notify_get_task();

  /* Start a process for a non-existent program. */

  printf("Start non-existent program...\n");

  const char *bad_argv[] = {"non_existent_program"};

  err = test_process(1, bad_argv);

  if (!err)
  {
    printf("Non-existent program was apparently successfully started.\n");
    return 1;
  }

  printf("Non-existent program result: %s\n", l4sys_errtostr(err));

  /* Start a process for the given program. */

  printf("Start process...\n");

  err = test_process(argc - 1, (const char **) argv + 1);

  if (err)
    return 1;

  notify_close(notifier);

  printf("End of test.\n");
  return 0;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
